Index: dl145/util/cbfstool/util.c
===================================================================
--- dl145.orig/util/cbfstool/util.c	2009-04-24 16:27:33.000000000 -0600
+++ dl145/util/cbfstool/util.c	2009-04-24 16:28:57.000000000 -0600
@@ -168,7 +168,8 @@
 }
 
 int create_rom(struct rom *rom, const unsigned char *filename,
-	       int romsize, int bootblocksize, int align)
+	       int romsize, const unsigned char *bootblockname,
+	       int bootblocksize, int align)
 {
 	unsigned char null = '\0';
 
@@ -180,8 +181,9 @@
 	/* Remember the size of the entire ROM */
 	rom->size = romsize;
 
-	/* The size of the archive section is everything but the bootblock */
-	rom->fssize = romsize - bootblocksize;
+	/* The size of the archive section is everything but the bootblock and
+	 * the cbfs master header. */
+	rom->fssize = romsize - bootblocksize - sizeof(struct cbfs_header);
 
 	/* Open the file descriptor */
 
@@ -201,29 +203,30 @@
 		return -1;
 	}
 
-	/* Clear the reset vector */
-	memset(rom->ptr + rom->size - 16, 0, 16);
-
-	ROM_WRITEL(rom, rom->size - 4,
-		   0xFFFFFFF0 - sizeof(struct cbfs_header));
-
+	printf("\tROM %p offset=%x\n", rom,
+	    ROM_PTR(rom, rom->size - 16 - bootblocksize - sizeof(struct cbfs_header))-ROM_PTR(rom,0));
 	/* This is a pointer to the header for easy access */
 	rom->header = (struct cbfs_header *)
-	    ROM_PTR(rom, rom->size - 16 - sizeof(struct cbfs_header));
-
+	    ROM_PTR(rom, rom->size - 16 - bootblocksize - sizeof(struct cbfs_header));
 	rom->header->magic = htonl(HEADER_MAGIC);
 	rom->header->romsize = htonl(romsize);
 	rom->header->bootblocksize = htonl(bootblocksize);
 	rom->header->align = htonl(align);
 	rom->header->offset = htonl(0);
 
+	add_bootblock(rom, bootblockname);
+
+	/* Clear the reset vector */
+	memset(rom->ptr + rom->size - 16, 0, 16);
+
+	ROM_WRITEL(rom, rom->size - 4,
+		   0xFFFFFFF0 - bootblocksize - sizeof(struct cbfs_header));
 	return 0;
 }
 
 int add_bootblock(struct rom *rom, const char *filename)
 {
 	unsigned int size;
-	//unsigned int offset;
 	int fd = size_and_open(filename, &size);
 	int ret;
 	struct cbfs_header tmp;
@@ -237,11 +240,7 @@
 		return -1;
 	}
 
-	/* Copy the current header into a temporary buffer */
-	memcpy(&tmp, rom->header, sizeof(struct cbfs_header));
-
 	/* Copy the bootblock into place at the end of the file */
-
 	ret = copy_from_fd(fd, ROM_PTR(rom, rom->size - ntohl(rom->header->bootblocksize)), size);
 
 	close(fd);
@@ -251,26 +250,6 @@
 		return -1;
 	}
 
-	/* FIXME: This should point to a location defined by coreboot */
-
-	ROM_WRITEL(rom, rom->size - 4,
-		   0xFFFFFFF0 - sizeof(struct cbfs_header));
-
-	/* This is a pointer to the header for easy access */
-	rom->header = (struct cbfs_header *)
-	    ROM_PTR(rom, rom->size - 16 - sizeof(struct cbfs_header));
-
-#if 0
-	/* Figure out the new location for the header */
-	offset = ROM_READL(rom, rom->size - 4);
-
-	rom->header = (struct cbfs_header *)
-	    ROM_PTR(rom, offset - (0xFFFFFFFF - rom->size));
-#endif
-
-	/* Replace the LAR header */
-	memcpy(rom->header, &tmp, sizeof(struct cbfs_header));
-
 	return 0;
 }
 
Index: dl145/util/cbfstool/cbfstool.h
===================================================================
--- dl145.orig/util/cbfstool/cbfstool.h	2009-04-24 16:27:33.000000000 -0600
+++ dl145/util/cbfstool/cbfstool.h	2009-04-24 16:27:34.000000000 -0600
@@ -58,7 +58,8 @@
 /* util.c */
 int open_rom(struct rom *rom, const char *filename);
 int create_rom(struct rom *rom, const unsigned char *filename, int size,
-	int bootblocksize,  int align);
+	       const unsigned char *bootblockname, int bootblocksize,
+	       int align);
 int size_and_open(const char *filename, unsigned int *size);
 int copy_from_fd(int fd, void *ptr, int size);
 int get_size(const char *size);
Index: dl145/util/cbfstool/create.c
===================================================================
--- dl145.orig/util/cbfstool/create.c	2009-04-24 16:27:33.000000000 -0600
+++ dl145/util/cbfstool/create.c	2009-04-24 16:27:34.000000000 -0600
@@ -59,11 +59,8 @@
 		return -1;
 	}
 
-	if (create_rom(rom, rom->name, size, bootblocksize, align))
+	if (create_rom(rom, rom->name, size, bootblock, bootblocksize, align))
 		return -1;
 
-	if (bootblock != NULL)
-		return add_bootblock(rom, bootblock);
-
 	return 0;
 }
